<?

/**
 * Builds an error array and kills the script returning a json encoded array
 * @param $error_id
 * @param string $additional_data Optional additional data to return with the error
 * @param boolean $return_string
 * @return null|string
 */
function return_error_as_json($error_id, $additional_data = null, $return_string = false)
{
  require_once __DIR__ . "/../internal/errors.php";
  global $con;
  $data = array();
  if (isset($errors))
  {
    if (key_exists($error_id, $errors))
    {
      $data["result"] = $error_id;
      if ($error_id == 1)
        $data["msg"] = $errors[$error_id] . ((isset($con)) ? mysqli_error($con) : "No SQL connection established");
      else
        $data["msg"] = $errors[$error_id];
    }
    else
    {
      $data["result"] = 9998;
      $data["msg"] = "Requested error not found in local error table";
    }
  }
  else
  {
    $data["result"] = 9999;
    $data["msg"] = "Error table not defined";
  }

  if (isset($additional_data))
    $data["data"] = $additional_data;
  if ($return_string)
    return json_encode($data);
  else
    die(json_encode($data));
}

/**
 * Builds the success response and kills the script, returning the response as json
 * @param array $data
 * @param boolean $return_string
 * @return null|string
 */
function return_success_as_json($data = null, $return_string = false)
{
  $return = array("result" => 0, "msg" => "");
  if (isset($data))
    $return = array_merge($return, $data);
  if ($return_string)
    return json_encode($return);
  else
    die(json_encode($return));
}

function parse_json_return($json_string)
{
  $return = json_decode($json_string, true);

  if ($return['result'] == 0)
    return array(true, 0, "");
  else
  {
    $data = array(false, $return['result'], $return['msg']);
    if(isset($return['data']))
      array_push($data, $return['data']);
    return $data;
  }
}

/**
 * Verifies that the provided id and secret is valid
 * Returns the client row on success, else kills script with appropriate error code
 * @param string $id client_id
 * @param string $secret client_secret
 * @return array
 */
function verify_client_id_secret($id, $secret)
{
  require_once __DIR__ . "/../inc_functions.php";
  require_once __DIR__ . "/../internal/inc_connect.php";
  require_once __DIR__ . "/../classes/ErrorCodes.php";
  global $con;

  $query = "SELECT * FROM client_keys WHERE client_id = '$id' AND client_secret = '$secret' AND type = 'api_auth'";
  $result = mysqli_query($con, $query);

  if (!$result)
    return_error_as_json(ErrorCodes::$SQL_ERROR, mysqli_error($con));
  elseif (mysqli_num_rows($result) == 1)
  {
    $row = mysqli_fetch_assoc($result);
    return $row;
  }
  else
    return_error_as_json(ErrorCodes::$INVALID_ID_OR_SECRET, array("client_id" => $id, "client_secret" => $secret));
}

/**
 * Verifies that the token exists, belongs with previously captured client and has not expired
 * Returns true on success, else kills script with appropriate error code
 * @param int $client_internal_id
 * @param string $token
 * @return bool
 */
function verify_client_token($client_internal_id, $token)
{
  require_once __DIR__ . "/../inc_functions.php";
  require_once __DIR__ . "/../internal/inc_connect.php";
  require_once __DIR__ . "/../classes/ErrorCodes.php";
  require_once __DIR__ . "/../api/inc_functions.php";
  global $con;

  $query = "SELECT * FROM client_keys WHERE token = '$token' AND type = 'api_auth'";
  $result = mysqli_query($con, $query);

  if (!$result)
    return_error_as_json(ErrorCodes::$SQL_ERROR, mysqli_error($con));
  elseif (mysqli_num_rows($result) == 1)
  {
    $key_data = mysqli_fetch_assoc($result);

    if ($key_data['id'] == $client_internal_id)
    {
      if ((time() - $key_data['token_refreshed']) > (60 * 60))
        return_error_as_json(ErrorCodes::$TOKEN_EXPIRED, time() - $key_data['token_refreshed']);

      $values = array(
        "last_accessed" => time(),
      );
      $result = mysqli_query($con, build_query("client_keys", $values, "update", "id", $key_data['id']));

      if (!$result)
        return_error_as_json(ErrorCodes::$SQL_ERROR, mysqli_error($con));
      else
        return true;
    }
  }
  return_error_as_json(ErrorCodes::$TOKEN_IS_INVALID);
  return false;
}