<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <title>API Help</title>
  <link rel='stylesheet' href='../assets/bootstrap.min.css' />
  <script src='../assets/jquery-1.12.0.min.js'></script>
  <script src='../assets/bootstrap.min.js'></script>
  <script src='../assets/cookie_functions.min.js'></script>
  <link rel='stylesheet' href='../assets/styles.css'/>
  <style>
    body
    {
      margin-top    : 25px;
      margin-bottom : 25px;
    }

    .path_div
    {
      color : white;
    }

    .path
    {
      color : gray;
    }

    h4
    {
      margin-bottom : 4px;
    }

    .description
    {
    }

    true
    {
      color       : green;
      font-weight : bold;
    }

    true::after
    {
      content : 'TRUE';
    }

    false
    {
      color       : red;
      font-weight : bold;
    }

    false::after
    {
      content : 'FALSE';
    }

    .row
    {
      padding-left        : 8px;
      transition-property : background-color;
      transition-duration : 1s;
    }

    .row:nth-child(2n)
    {
      background-color : rgba(0, 0, 0, 0.15);
    }

    .row:hover
    {
      background-color : rgba(0, 24, 57, 0.65);
    }

    a
    {
      text-decoration : none;
    }

    .hl
    {
      font-weight      : bold;
      color            : white;
      background-color : rgba(21,0,76,0.3);
    }
  </style>
</head>
<?
require_once "../internal/errors.php";
require_once "../classes/ErrorCodes.php";
require_once "../inc_functions.php";
require_once "inc_common_functions.php";
global $errors;

$endpoints = array(
  array(
    "path" => array("api", "{api_ver}", "current_tag"),
    "versions" => array("v1"),
    "description" => "Gets the last pushed tag (Current version).",
    "parameters" => array(),
    "returns" => array(
      "major" => array("STRING", "Latest major tag (Requires client update)"),
      "minor" => array("STRING", "Latest minor tag (Server-only changes)"),
    ),
    "errors" => array(1,11),
    "requires_auth" => false,
    "requires_key" => false,
    "requires_post" => false,
    "rate_limit" => "none"),

  array(
    "path" => array("api", "{api_ver}", "auth"),
    "versions" => array("v1"),
    "description" => "Acquire an auth token for use with requests that require it.",
    "parameters" => array(
      "client_id" => array("STRING", true, ""),
      "client_secret" => array("STRING", true, "")
    ),
    "returns" => array(
      "token" => array("STRING", "Session token valid for up to 1 hour after request")
    ),
    "requires_auth" => false,
    "requires_key" => true,
    "requires_post" => true,
    "rate_limit" => "once per hour"
  ),

  array(
    "path" => array("api", "{api_ver}", "get_download_url"),
    "versions" => array("v1"),
    "description" => "Get the url for downloading the full modpack or a patch. Triggers generation if necessary",
    "parameters" => array(
      "client_id" => array("STRING", true, ""),
      "client_secret" => array("STRING", true, ""),
      "token" => array("STRING", true, ""),
      "patch_from" => array("STRING", false, "A tag to patch from"),
      "opt_mods" => array("ARRAY", false, "An array of optional mods to include")
    ),
    "returns" => array(
      "id" => array("NUMBER", "The id to pass on to api/get_file/"),
      "filename" => array("STRING", "The filename that the file would have been given had it been downloaded through the GUI"),
      "is_patch" => array("BOOL", ""),
      "size_raw" => array("NUMBER", "The file size in bytes"),
      "size" => array("STRING", "The file size in human-readable format, eg 12.5MB"),
      "mod_count" => array("NUMBER", ""),
      "mod_count_opt" => array("NUMBER", ""),
      "res_count" => array("NUMBER", "Resource pack count")
    ),
    "errors" => array(1,2,3,4,5,6,7,8),
    "requires_auth" => true,
    "requires_key" => true,
    "requires_post" => true,
    "rate_limit" => "No Limit"
  ),

  array(
    "path" => array("api", "{api_ver}", "get_file"),
    "versions" => array("v1"),
    "description" => "Download an archive by id",
    "parameters" => array(
      "client_id" => array("STRING", true, ""),
      "client_secret" => array("STRING", true, ""),
      "token" => array("STRING", true, ""),
      "id" => array("NUMBER", true, "The id of the file to download, provided by api/get_download_url/")
    ),
    "returns" => array(
      "file" => array("FILE", "The requested file"),
      "error" => array("STRING", "On failure the error is returned as a plaintext string")
    ),
    "errors" => array(1,2,3,4,5,6,7,8,9),
    "requires_auth" => true,
    "requires_key" => true,
    "requires_post" => true,
    "rate_limit" => "No Limit"
  ),

  array(
    "path" => array("api", "{api_ver}", "server_message"),
    "versions" => array("v1"),
    "description" => "Get a random server message (Same as the ones displayed in-game)",
    "parameters" => array(
      "ignore_time_limit" => array("STRING", false, "Should be 'true' to operate. Ignores the per-message time limit if set")
    ),
    "returns" => array(
      "color" => array("STRING", "The messages color name"),
      "text" => array("STRING", "The message")
    ),
    "requires_auth" => false,
    "requires_key" => false,
    "requires_post" => false,
    "rate_limit" => "No Limit"
  ),

  array(
    "path" => array("api", "{api_ver}", "test_token"),
    "versions" => array("v1"),
    "description" => "Test your token to see if it's still valid",
    "parameters" => array(
      "client_id" => array("STRING", true, ""),
      "client_secret" => array("STRING", true, ""),
      "token" => array("STRING", true, "")
    ),
    "returns" => array(),
    "errors" => array(1,2,3,4,5,6,7,8),
    "requires_auth" => true,
    "requires_key" => true,
    "requires_post" => true,
    "rate_limit" => "No Limit"
  ),

  array(
    "path" => array("api", "{api_ver}", "get_opt_mods"),
    "versions" => array("v1"),
    "description" => "Get a list of optional mods that can be provided to get_download_url",
    "parameters" => array(),
    "returns" => array(
      "mods" => array("ARRAY", "An array of optional mods (or an empty array)")
    ),
    "errors" => array(5),
    "requires_auth" => false,
    "requires_key" => false,
    "requires_post" => false,
    "rate_limit" => "No Limit"
  ),

  array(
    "path" => array("api", "{api_ver}", "get_error_message"),
    "versions" => array("v1"),
    "description" => "Get a specific message from an error code, or get an array of all the error messages",
    "parameters" => array(
      "error_code" => array("INT", false, "An error code"),
    ),
    "returns" => array(
      "error_message" => array("STRING|ARRAY", "Either a single error message or an array of all error messages")
    ),
    "errors" => array(),
    "requires_auth" => false,
    "requires_key" => false,
    "requires_post" => false,
    "rate_limit" => "No Limit"
  ),

  array(
    "path" => array("api", "{api_ver}", "get_ads"),
    "versions" => array("v1"),
    "description" => "Gets one or more ads of all or specified types",
    "parameters" => array(
      "client_id" => array("STRING", true, ""),
      "client_secret" => array("STRING", true, ""),
      "token" => array("STRING", true, ""),
      "type" => array("STRING", false, "Valid types are 'any', 'buy' and 'sell' - Default: 'any'"),
      "limit" => array("NUMBER", false, "Limit number of ads returned - Default: -1 (No limit)"),
    ),
    "returns" => array(
      "ads" => array("ARRAY", "A numeric array containing the fetched ads which in turn contain the following fields:"),
      "id" => array("NUMBER", "The incremental id of the ad"),
      "created_time" => array("NUMBER", "Unix timestamp of the creation time of the ad"),
      "bumped" => array("NUMBER", "Unix timestamp of the last bump of the ad"),
      "edited" => array("NUMBER", "Unix timestamp of the last edit"),
      "type" => array("STRING", "The type of the ad ('buy' or 'sell')"),
      "title" => array("STRING", "The title of the ad"),
      "body" => array("STRING", "The body of the ad"),
      "minecraft_name" => array("STRING", "The minecraft name of the ad owner"),
      "minecraft_uuid" => array("STRING", "The minecraft uuid of the ad owner (You could use this to fetch their head texture or similar from some service)"),
    ),
    "errors" => array(),
    "requires_auth" => true,
    "requires_key" => true,
    "requires_post" => true,
    "rate_limit" => "No limit"
  ),
)
?>
<body>
<div class="container">
  <?
  $menu = array(
    "New Client" => array("type" => "link", "url" => "new_client"),
    "Changelog" => array("type" => "link", "url" => "changelog"),
    "Back to Modpack" => array("type" => "link", "url" => ".."),
  );

  build_menu();
  ?>
  <h4>General API information:</h4><br/>
  <ul>
    <li>GET endpoints can be called using POST as well.</li>
    <li>POST endpoints can only be called with POST.</li>
    <li>Replies to requests will always be in JSON and always contains the fields <code>'result'</code> and <code>'msg'</code>.</li>
    <li>If a request is successful <code>'result'</code> is 0 and <code>'msg'</code> is an empty string. Additional fields will be sent according to the endpoints specification.</li>
    <li>If an error occurs <code>'result'</code> will be non-zero (error code) and <code>'msg'</code> contains the error message.</li>
    <li>Some errors may send additional information with the <code>'data'</code> field, such as on SQL errors. Generally this field is omitted.</li>
  </ul>

  <h4>Errors:</h4>
  <div id="all_errors" style="display: none;">
    <table>
    <?
    foreach ($errors as $code => $error)
    {
      if ($error != "")
        echo "<tr><td>" . $code . "</td><td>\"" . $error . "\"</td></tr>";
    }
    ?>
    </table>
  </div>
  <div style="margin-bottom: 20px;"><span style="cursor: pointer;" onclick="
    var elem = document.getElementById('all_errors');
    if (elem.style.display == 'none') {
      elem.style.display = null;
      this.innerHTML = 'Hide Errors';
    } else {
      elem.style.display = 'none';
      this.innerHTML = 'Show Errors';
    }
    ">Show Errors</span></div>

  <h4>Endpoints:</h4>
  <?

  foreach ($endpoints as $point)
  {
    $path_raw = "";
    $path_str = "";
    foreach ($point['path'] as $path)
    {
      $path_raw .= $path . "/";
      $path_str .= "<span class='path'>$path</span><span class='path_div'>/</span>";
    }
    $path_str = insert($path_str, array("api_ver" => $point["versions"][count($point["versions"]) - 1]));
    $path_raw = insert($path_raw, array("api_ver" => $point["versions"][count($point["versions"]) - 1]));
    $path_str = "<a href='#$path_raw'>" . $path_str . "</a>";
    ?>
    <div class="row" style="margin-top: 25px;">
      <div id="<?=$path_raw?>"><h4><?= ($point['requires_post']) ? "POST" : "GET" ?> <?= $path_str ?></h4></div>
      <div class="description">Available in versions: <? $str = ""; foreach ($point['versions'] as $ver) $str .= $ver . ", "; echo trim($str, ", "); ?></div>
      <div class="description"><?= $point['description'] ?></div>
      <div>
        <dim>Requires Auth:</dim> <?= $point['requires_auth'] ? "<true></true>" : "<false></false>" ?></div>
      <div>
        <dim>Requires Key:</dim> <?= $point['requires_key'] ? "<true></true>" : "<false></false>" ?></div>
      <div>
        <div class="hl">Parameters:</div>
        <?
        $parameter_str = "";
        foreach ($point['parameters'] as $parameter => $value)
          $parameter_str .= "<div class='row'><div class='two columns'>" . $parameter . "</div><div class='ten columns'>Type: " . $value[0] . ($value[1] ? " - Required" : " - Optional") . " <dim>" . $value[2] . "</dim></anchor></div></div>\n";
        if ($parameter_str == "")
          $parameter_str = "<dim>None</dim>";
        echo $parameter_str;
        ?>
      </div>
      <div>
        <div class="hl">Returns:</div>
        <?
        $returns_str = "";
        foreach ($point['returns'] as $return => $value)
          $returns_str .= "<div class='row'><div class='two columns'>" . $return . "</div><div class='ten columns'>Type: " . $value[0] . " <dim>" . $value[1] . "</dim></div></div>\n";
        if ($returns_str == "")
          $returns_str = "<dim>Just the default fields</dim>";
        echo $returns_str;
        ?>
      </div>
      <div>
        <div class="hl">Errors:</div>
        <?
        $file_path = "../" . preg_replace('/\/$/', ".php", $path_raw);
        $file = file_get_contents($file_path);
        $point_errors = array();
        if ($file)
        {
          $match_count = preg_match_all('/ErrorCodes::\$([A-Z_]+),?.*?\).*;/', $file, $matches, PREG_SET_ORDER, 0);

          foreach ($matches as $match)
          {
            if (!in_array(ErrorCodes::$ERRORS[$match[1]], $point_errors))
              $point_errors[] = ErrorCodes::$ERRORS[$match[1]];
          }

          if (preg_match_all('/verify_client_id_secret\(/', $file, $matches))
          {
            if (!in_array(ErrorCodes::$SQL_ERROR, $point_errors))
              array_push($point_errors, ErrorCodes::$SQL_ERROR);
            if (!in_array(ErrorCodes::$INVALID_ID_OR_SECRET, $point_errors))
              array_push($point_errors, ErrorCodes::$INVALID_ID_OR_SECRET);
          }

          if (preg_match_all('/verify_client_token\(/', $file, $matches))
          {
            if (!in_array(ErrorCodes::$TOKEN_EXPIRED, $point_errors))
              array_push($point_errors, ErrorCodes::$TOKEN_EXPIRED);
            if (!in_array(ErrorCodes::$SQL_ERROR, $point_errors))
              array_push($point_errors, ErrorCodes::$SQL_ERROR);
            if (!in_array(ErrorCodes::$TOKEN_IS_INVALID, $point_errors))
              array_push($point_errors, ErrorCodes::$TOKEN_IS_INVALID);
          }
          sort($point_errors);
        }
        else
          echo "<div>Unable to get file " . realpath($file_path) . " (" . $file . ")</div>";

        $returns_str = "";
        foreach ($point_errors as $value)
        {
          if (isset($errors[$value]))
            $returns_str .= "<div class='row'><div class='two columns'>" . $value . "</div><div class='ten columns'>" . $errors[$value] . "</div></div>\n";
        }
        if ($returns_str == "")
          $returns_str = "<div class='row'><div class='two columns'>None</div><div class='eight columns dim'>What could possibly go wrong?</div></div>";
        echo $returns_str;
        ?>
      </div>
    </div> <?
  }
  ?>
</div>
</body>
</html>