<?


namespace Patreon;

class SettingType
{
  public static $STRING = "string";
  public static $NUMBER = "number";
  public static $SWITCH = "switch";
}

/**
 * Class Setting
 * @package Patreon
 * @property String $key
 * @property String $title
 * @property String $type
 * @property String $description
 * @property String $value
 */
abstract class Setting
{
  public $key;
  public $title;
  public $type;
  public $description;
  public $value;

  /**
   * @return Setting|SettingNumber|SettingString|SettingSwitch
   */
  public function Load()
  {
    global $con;
    require_once __DIR__ . "/../internal/inc_connect.php";
    if (!isset($con))
    {
      log_file("No SQL connection for loading setting '" . $this->key . "'", "error.log");
      return $this;
    }
    $setting_id = $this->key;
    $query = "SELECT * FROM misc WHERE field_key = '$setting_id'";
    $result = mysqli_query($con, $query);
    if (!$result)
    {
      log_file("SQL Error loading setting '" . $this->key . "':", "error.log");
      log_file(mysqli_error($con));
      return $this;
    }
    if (mysqli_num_rows($result) == 1)
    {
      $setting = mysqli_fetch_assoc($result);
      $this->value = $setting['field_value'];
    }
    return $this;
  }

  /**
   * @return void
   * @throws \SQLConnectException
   * @throws \SQLQueryException
   */
  public function Save()
  {
    global $con;
    require_once __DIR__ . "/../internal/inc_connect.php";
    require_once "inc_common_functions.php";
    if (!isset($con))
      throw new \SQLConnectException("No SQL connection");
    $setting_id = $this->key;
    $value = $this->value;
    $query = "INSERT INTO misc (field_key, field_value) VALUES ('$setting_id', '$value') ON DUPLICATE KEY UPDATE field_value = '$value'";
    $result = mysqli_query($con, $query);
    if (!$result)
    {
      $error = "Failed to save '$setting_id'";
      $sql_error = mysqli_error($con);
      log_file($sql_error, "../sql.log");
      throw new \SQLQueryException($error, $query, $sql_error);
    }
  }

  /**
   * @param bool $saveToDatabase Send false to prevent writing to database (used for local values like switches)
   * @return String
   * @throws \SQLConnectException
   * @throws \SQLQueryException
   */
  public function GetValueFromPost($saveToDatabase = true)
  {
    $valueChanged = false;
    if (isset($_POST[$this->key]))
    {
      if ($this->value !== $_POST[$this->key])
        $valueChanged = true;
      $this->value = $_POST[$this->key];
      if ($saveToDatabase && $valueChanged)
        $this->Save();
    }
    return $this->value;
  }

  /**
   * @return string
   */
  public function GetValueFromPostReadOnly()
  {
    try
    {
      return $this->GetValueFromPost(false);
    }
    catch (\Exception $e)
    {
      return "";
    }
  }

  /**
   * @param String $value
   * @return bool
   */
  public function Test($value)
  {
    return $this->value === $value;
  }

  /**
   * @return String
   */
  public function Value()
  {
    return $this->value;
  }

  public function GetControl() {}
}

class SettingString extends Setting
{
  /**
   * SettingString constructor.
   * @param String $key
   * @param String $title
   * @param String $description
   * @param String $value
   */
  public function __construct($key, $title, $description = "", $value = "")
  {
    $this->key = $key;
    $this->title = $title;
    $this->description = $description;
    $this->type = SettingType::$STRING;
    $this->value = $value;
  }

  public function GetControl()
  {
    return "<input class=\"form-control\" type=\"text\" name=\"" . $this->key . "\" value=\"" . $this->value . "\" placeholder=\"" . $this->title . "\"/>";
  }
}

/**
 * Class SettingNumber
 * @package Patreon
 * @property Int $min
 * @property Int $max
 * @property Double $step
 */
class SettingNumber extends Setting
{
  public $min;
  public $max;
  public $step;

  /**
   * SettingNumber constructor.
   * @param String $key
   * @param String $title
   * @param String $description
   * @param String $value
   * @param Int $min
   * @param Int $max
   * @param Double $step
   */
  public function __construct($key, $title, $description = "", $value = "0", $min = null, $max = null, $step = 1)
  {
    $this->key = $key;
    $this->title = $title;
    $this->description = $description;
    $this->value = $value;
    $this->type = SettingType::$NUMBER;
    $this->min = $min;
    $this->max = $max;
    $this->step = $step;
  }

  public function GetControl()
  {
    $min = (isset($this->min) ? "min=\"" . $this->min . "\"" : "");
    $max = (isset($this->max) ? "max=\"" . $this->max . "\"" : "");
    $step = (isset($this->step) ? "step=\"" . $this->step . "\"" : "");
    return "<input class=\"form-control\" type=\"number\" name=\"" . $this->key . "\" value=\"" . $this->value . "\" placeholder=\"" . $this->title . "\" $min $max $step/>";
  }
}

/**
 * Class SettingBool
 * @package Patreon
 * @property String[] $options ex array("value" => "Label", "value2" => "Label 2", ...)
 * @property String[]|string $option_styles ex "style" or array( "value" => "style", "value2" => "style2", ...) where value is equivalent to value in $options property
 */
class SettingSwitch extends Setting
{
  public $options;
  public $option_styles;

  /**
   * SettingSwitch constructor.
   * @param String $key
   * @param String $title
   * @param String $description
   * @param String $value
   * @param String[] $options
   * @param String[]|string $option_styles
   */
  public function __construct($key, $title, $description = "", $value = "enabled", $options = array("enabled" => "Enabled", "disabled" => "Disabled"), $option_styles = null)
  {
    $this->key = $key;
    $this->title = $title;
    $this->description = $description;
    $this->type = SettingType::$SWITCH;
    $this->value = $value;
    $this->options = $options;
    $this->option_styles = $option_styles;
  }

  public function GetControl()
  {
//    return generate_settings_switch($this);
    $str = "<div class='btn-group btn-group-toggle' role='group' aria-label='" . $this->title . "'>";
    $counter = 0;
    foreach ($this->options as $key => $label)
    {
      $sel = $this->value == $key;
      $style = "primary";
      if (isset($this->option_styles))
      {
        if (is_string($this->option_styles))
          $style = $this->option_styles;
        elseif (is_array($this->option_styles) && isset($this->option_styles[$key]))
          $style = $this->option_styles[$key];
      }
      $str .= "<label onclick='button_group_activate(event)' class='btn btn-$style " . ($sel ? "active" : "") . "'><input type='radio' name='" . $this->key . "' id='" . $this->key . "_$counter' value='" . $key . "' " . ($sel ? "checked" : "") . " >" . $label . "</label>";
      $counter++;
    }
    $str .= "</div>";
    return $str;
  }

  /**
   * @return bool
   */
  public function TestEnabled()
  {
    return $this->value === "enabled";
  }

  /**
   * @return bool
   */
  public function TestDisabled()
  {
    return $this->value === "disabled";
  }
}