<?

use Patreon\Setting;

require_once __DIR__ . "/Setting.php";

class SimpleSetting
{
  public static $MAINTENANCE_MODE = "maintenance_mode";
  public static $PAGE_TITLE = "page_title";
  public static $FORGE_VERSION = "forge_version";
  public static $DOWNLOADS_ENABLED = "downloads_enabled";
  public static $USER_LOGIN = "user_login";
  public static $MEDIA_STREAM_FOLLOWER = "media_stream_follower";
  public static $MEDIA_STREAM_VIEWS = "media_stream_views";
  public static $MEDIA_VIDEO_FOLLOWER = "media_video_follower";
  public static $MEDIA_VIDEO_VIEWS = "media_video_views";
  public static $MEDIA_NOTE = "media_note";
  public static $WHITELIST_UPDATE = "whitelist_update";
  public static $PUSH_NOTIFICATIONS = "push_notifications";
  public static $MIN_TIME_BEFORE_REPEAT = "min_time_before_repeat";
  public static $PUBLIC_MOD_LIST_ENABLED = "public_mod_list_enabled";
  public static $TEST_SETTING = "test_setting";

  public $labels = array();

  /**
   * @param Setting $setting
   */
  public function AddSetting($setting)
  {
    $this->labels[$setting->key] = $setting;
  }

  /**
   * SimpleSetting constructor.
   * Used when needing to iterate over settings
   */
  public function __construct() {
    $this->labels = array();
      //$this::$MAINTENANCE_MODE => array("Maintenance Mode", ""),
      //$this::$PAGE_TITLE => array("Page Title", "Title displayed on windows/tabs"),
      //$this::$FORGE_VERSION => array("Forge Version", ""),
      //$this::$DOWNLOADS_ENABLED => array("Downloads Enabled", ""),
      //$this::$USER_LOGIN => array("User Login", ""),
      //$this::$MEDIA_STREAM_FOLLOWER => array("Streaming Followers", ""),
      //$this::$MEDIA_STREAM_VIEWS => array("Streaming Views", ""),
      //$this::$MEDIA_VIDEO_FOLLOWER => array("Video Followers", ""),
      //$this::$MEDIA_VIDEO_VIEWS => array("Video Views", ""),
      //$this::$MEDIA_NOTE => array("Media Note", ""),
      //$this::$WHITELIST_UPDATE => array("Whitelist Update", ""),
      //$this::$PUSH_NOTIFICATIONS => array("Push Notifications", ""),
      //$this::$MIN_TIME_BEFORE_REPEAT => array("Min time before repeat", ""),
      //$this::$PUBLIC_MOD_LIST_ENABLED => array("Public Mod List Enabled", ""),
    $this->AddSetting(new \Patreon\SettingSwitch($this::$MAINTENANCE_MODE, "Maintenance Mode", "Locks down site and prevents downloading the pack and uploading or changing files."));
    $this->AddSetting(new \Patreon\SettingString($this::$PAGE_TITLE, "Page Title", "Title displayed on windows/tabs", "Modpack"));
    $this->AddSetting(new \Patreon\SettingString($this::$FORGE_VERSION, "Forge Version", "The Forge version on the server"));
    $this->AddSetting(new \Patreon\SettingSwitch($this::$DOWNLOADS_ENABLED, "Downloads Enabled"));
    $this->AddSetting(new \Patreon\SettingSwitch($this::$USER_LOGIN, "Require Login"));
    $this->AddSetting(new \Patreon\SettingNumber($this::$MEDIA_STREAM_FOLLOWER, "Stream Followers"));
    $this->AddSetting(new \Patreon\SettingNumber($this::$MEDIA_STREAM_VIEWS, "Stream Views"));
    $this->AddSetting(new \Patreon\SettingNumber($this::$MEDIA_VIDEO_FOLLOWER, "Video Followers"));
    $this->AddSetting(new \Patreon\SettingNumber($this::$MEDIA_VIDEO_VIEWS, "Video Views"));
    $this->AddSetting(new \Patreon\SettingString($this::$MEDIA_NOTE, "Media Note", "Note displayed along with media requirements"));
    $this->AddSetting(new \Patreon\SettingSwitch($this::$WHITELIST_UPDATE, "Whitelist Update", "Automatically update whitelist via FTP"));
    $this->AddSetting(new \Patreon\SettingSwitch($this::$PUSH_NOTIFICATIONS, "Push Notifications", "Publish mod updates to PushBullet (WIP)"));
    $this->AddSetting(new \Patreon\SettingNumber($this::$MIN_TIME_BEFORE_REPEAT, "Min Time Before Repeat", "Min time between repeat server messages from API"));
    $this->AddSetting(new \Patreon\SettingSwitch($this::$PUBLIC_MOD_LIST_ENABLED, "Public Mod List"));
  }

  /**
   */
  public function LoadAll()
  {
    foreach ($this->labels as $setting)
      $this->Load($setting->key);
  }

  /**
   * @param $key
   * @return Setting|\Patreon\SettingSwitch|\Patreon\SettingString|\Patreon\SettingNumber
   */
  public function Load($key)
  {
    /** @var Setting $setting */
    $setting = $this->labels[$key];
    return $setting->Load();
  }

  /**
   * @param String $key
   * @return Patreon\SettingString|\Patreon\SettingNumber|\Patreon\SettingSwitch
   */
  public function Get($key)
  {
    return $this->labels[$key];
  }

  public function ProcessAllFromPost()
  {
    /** @var Setting $setting */
    foreach ($this->labels as $setting)
      $setting->GetValueFromPost();
  }

  public function GetControlRows($type = null)
  {
    $str = "";
    /** @var Setting $setting */
    foreach ($this->labels as $setting)
    {
      if (isset($type) && $setting->type !== $type)
        continue;
      $cols = "8";
      if ($setting->type === \Patreon\SettingType::$NUMBER)
        $cols = "3";
      $str .= "<div class=\"row\">
        <div class=\"col-md-4\">" . $setting->title . "<br/><span style='color: gray;'>" . $setting->description . "</span></div>
        <div class=\"col-md-$cols\">
          " . $setting->GetControl() . "
        </div>
      </div>";
    }
    return $str;
  }
}
