<?

function debug_init()
{
  require_once __DIR__ . "/internal/inc_connect.php";
  ini_set('display_errors', 1);
  error_reporting(E_ALL ^ E_NOTICE);
}

function db_init()
{
  global $con;
  require_once __DIR__ . "/internal/inc_connect.php";
  if (isset($con))
  {
    log_file("Create files table: " . mysqli_query($con, "CREATE TABLE IF NOT EXISTS files (id INT AUTO_INCREMENT PRIMARY KEY, mod_name VARCHAR(255), mod_id VARCHAR(255), system_file_name VARCHAR(255), filename VARCHAR(255), parent_identifier VARCHAR(255), file_type VARCHAR(255), path VARCHAR(255), update_id VARCHAR(255), update_time INT, version_from VARCHAR(255), version_to VARCHAR(255), is_required TINYINT, pack_side TINYINT, other_dependencies VARCHAR(255), default_selected TINYINT, update_reason VARCHAR(500), release_tag VARCHAR(50))"), "sql.log");
    log_file("Create users table: " . mysqli_query($con, "CREATE TABLE IF NOT EXISTS users (id INT AUTO_INCREMENT PRIMARY KEY, email VARCHAR(255) UNIQUE, role VARCHAR(20), login_token VARCHAR(255), token_expires INT, minecraft_name VARCHAR(255), minecraft_uuid VARCHAR(36), note VARCHAR(255))"), "sql.log");
    log_file("Create builds table: " . mysqli_query($con, "CREATE TABLE IF NOT EXISTS builds (id INT AUTO_INCREMENT PRIMARY KEY, hash VARCHAR(128), system_file_name VARCHAR(255), filename VARCHAR(255), creation_time INT, last_download INT, mod_count INT, mod_opt_count INT, res_count INT, patch_from VARCHAR(255), patch_to VARCHAR(255), is_patch TINYINT)"), "sql.log");
    log_file("Create misc data table: " . mysqli_query($con, "CREATE TABLE IF NOT EXISTS misc (field_key VARCHAR(255) PRIMARY KEY, field_value VARCHAR(255))"), "sql.log");
    log_file("Create server messages table: " . mysqli_query($con, "CREATE TABLE IF NOT EXISTS server_messages (id int not null auto_increment primary key, color varchar(50) default '' null, message varchar(255) null, enabled tinyint(1) default '1' null, constraint server_messages_message_uindex unique (message), last_request INT DEFAULT 0)"), "sql.log");
    log_file("Create tags table: " . mysqli_query($con, "CREATE TABLE IF NOT EXISTS tags (id INT AUTO_INCREMENT PRIMARY KEY, tag VARCHAR(255), created_time INT NOT NULL, low_entry INT, high_entry INT, tag_type VARCHAR(255))"), "sql.log");
    log_file("Create client_keys table: " . mysqli_query($con, "CREATE TABLE IF NOT EXISTS client_keys (id INT AUTO_INCREMENT PRIMARY KEY, app_name VARCHAR(255), app_note VARCHAR(255), client_id VARCHAR(36), client_secret VARCHAR(255), client_owner VARCHAR(255), created_time INT NOT NULL, token VARCHAR(255), token_refreshed INT NOT NULL, last_accessed INT NOT NULL, type VARCHAR(255))"), "sql.log");
    log_file("Create ads table: " . mysqli_query($con, "CREATE TABLE IF NOT EXISTS ads (id INT AUTO_INCREMENT PRIMARY KEY, created_time INT NOT NULL, bumped INT, edited INT, owner_email VARCHAR(255), type VARCHAR(255), title VARCHAR(150), body TEXT)"), "sql.log");
  }
  else
    die("<body style='background-color: black; color: white;'>ERROR: Database connection not established!</body>");
}

function verify_token($token, $email)
{
  global $con;
  require_once __DIR__ . "/internal/inc_connect.php";
  $query = "SELECT * FROM users WHERE email = '$email' && login_token = '$token'";
  $result = mysqli_query($con, $query);

  if ($result && mysqli_num_rows($result) == 1)
    return true;
  return false;
}

function fileCodeToMessage($code)
{
  switch ($code) {
    case UPLOAD_ERR_INI_SIZE:
      $message = "The uploaded file exceeds the upload_max_filesize directive in php.ini";
      break;
    case UPLOAD_ERR_FORM_SIZE:
      $message = "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form";
      break;
    case UPLOAD_ERR_PARTIAL:
      $message = "The uploaded file was only partially uploaded";
      break;
    case UPLOAD_ERR_NO_FILE:
      $message = "No file was uploaded";
      break;
    case UPLOAD_ERR_NO_TMP_DIR:
      $message = "Missing a temporary folder";
      break;
    case UPLOAD_ERR_CANT_WRITE:
      $message = "Failed to write file to disk";
      break;
    case UPLOAD_ERR_EXTENSION:
      $message = "File upload stopped by extension";
      break;

    default:
      $message = "Unknown upload error";
      break;
  }
  return $message;
}

/**
 * Inserts array of values into MYSQL query properly wherever the {values} tag is
 * @param $query string
 * @param $values array
 * @return string
 * @deprecated Superseded by build_query
 */
function value_insert($query, $values)
{
  $value_str = "";
  foreach ($values as $key => $value)
  {
    if (stristr($key, "#") === false)
      $value_str .= $value."','";
  }
  $value_str = substr_replace($value_str, "", -3, 3);
  return str_replace("'{values}'", "'".$value_str."'", $query);
}

/**
 * Builds a statement from the given values
 * If type is "update" where_column and where_value are required, otherwise they are ignored
 * @param string $table
 * @param array $values
 * @param string $type
 * @param string $where_column
 * @param string $where_value
 * @return string
 */
function build_query($table, $values, $type = "insert", $where_column = null, $where_value = null)
{
  $query = "FALSE";
  if ($type == "insert")
  {
    $query = "INSERT INTO $table (";
    foreach ($values as $key => $value)
      $query .= "$key, ";
    $query = trim($query, ", ");
    $query .= ") VALUES (";
    foreach ($values as $value)
      $query .= "'$value', ";
    $query = trim($query, ", ");
    $query .= ");";
  }
  elseif ($type == "update" && isset($where_column) && isset($where_value))
  {
    $query = "UPDATE $table SET ";
    foreach ($values as $key => $value)
      $query .= "$key = '$value', ";
    $query = trim($query, ", ");
    $query .= " WHERE $where_column = '$where_value';";
  }
  elseif ($type == "auto")
  {
    $query = "INSERT INTO $table (";
    foreach ($values as $key => $value)
      $query .= "$key, ";
    $query = trim($query, ", ");
    $query .= ") VALUES (";
    foreach ($values as $value)
      $query .= "'$value', ";
    $query = trim($query, ", ");
    $query .= ") ON DUPLICATE KEY UPDATE ";
    foreach ($values as $key => $value)
      if ($value != '')
        $query .= "$key = '$value', ";
    $query = trim($query, ", ");
    $query .= ";";
  }
  return $query;
}

/**
 * @param array $array
 * @param number $a
 * @param number $b
 * @param string $title
 * @param bool $optional
 */
function draw_elements($array, $a, $b, $title, $optional = false)
{
  global $mod_element_req, $mod_element_opt;
  require_once "inc_common_functions.php";
  log_file("Staring at index $a for $b items");

  if ($optional)
    $element = $mod_element_opt;
  else
    $element = $mod_element_req;

  echo "<div class='container'>";
  echo "<div class='title'>$title</div>";
  for ($i = $a; $i < ($a + $b) && $i < count($array); $i++)
  {
    if ($array[$i]['display'] != "No Match" && $array[$i]['optional'] == $optional)
    {
      echo insert($element, array($i, $array[$i]['display'], $array[$i]['file'] . " (" . humanFileSize($array[$i]['size']) . ")", $array[$i]['dir'] . $array[$i]['file']));
    }
    else
      $b++;
  }
  echo "</div>";
}

/**
 * http://stackoverflow.com/questions/15188033/human-readable-file-size
 * @param $size
 * @return string
 */
function humanFileSize($size)
{
  if ($size >= 1073741824)
  {
    $fileSize = round($size / 1024 / 1024 / 1024, 1) . 'GB';
  }
  elseif ($size >= 1048576)
  {
    $fileSize = round($size / 1024 / 1024, 1) . 'MB';
  }
  elseif ($size >= 1024)
  {
    $fileSize = round($size / 1024, 1) . 'KB';
  }
  else
  {
    $fileSize = $size . ' bytes';
  }
  return $fileSize;
}

function array_sort($array)
{
  if (!function_exists('rank_strings'))
  {
    function rank_strings($a, $b)
    {
      $value = strnatcmp(strtolower($a["display"]), strtolower($b["display"]));
      return $value;
    }
  }

  usort($array, 'rank_strings');
  return $array;
}

function conceal_email($email)
{
  $email = explode("@", $email);
  return substr_replace($email[0], "****", 4) . "@".$email[1];
}

/**
 * @param $setting_id
 * @param bool $read_only
 * @deprecated Use SimpleSetting and Setting instead
 */
function handle_setting($setting_id, $read_only = false)
{
  global $_POST, $con, $settings, $error;
  require_once __DIR__ . "/internal/inc_connect.php";
  $value = "";
  if (!$read_only)
  {
    if (isset($_POST[$setting_id]))
    {
      $value = $_POST[$setting_id];
      $query = "INSERT INTO misc (field_key, field_value) VALUES ('$setting_id', '$value') ON DUPLICATE KEY UPDATE field_value = '$value'";
      $result = mysqli_query($con, $query);
      if (!$result)
      {
        $error = "Failed to update $setting_id";
        log_file(mysqli_error($con), "../sql.log");
      }
    }
  }

  $query = "SELECT * FROM misc WHERE field_key = '$setting_id'";
  $result = mysqli_query($con, $query);
  if ($result && mysqli_num_rows($result) == 1)
  {
    $setting = mysqli_fetch_assoc($result);
    $value = $setting['field_value'];
  }
  $settings[$setting_id] = $value;
}

/**
 * @param $setting
 * @param $value
 * @return bool
 * @deprecated Use SimpleSetting system instead
 */
function test_setting($setting, $value)
{
  global $settings;
  if (isset($settings) && isset($settings[$setting]) && $settings[$setting] == $value)
    return true;
  return false;
}

function mysqli_escape($con, $string)
{
  $new_string = mysqli_escape_string($con, $string);
  if ($new_string != "")
    return $new_string;
  return $string;
}

/**
 * @param bool $ignore_tags Just makes it return "unknown_tag"???
 * @param bool $include_update_time Appends update time (Y-m-d H:i) after tag
 * @param bool $include_time Whether to append time after date
 * @return string
 */
function get_latest_tag($ignore_tags = false, $include_update_time = false, $include_time = true)
{
  global $con;
  require_once __DIR__ . "/internal/inc_connect.php";
  $tag = "unknown_tag";
  if (isset($con) && !$ignore_tags)
  {
    $result = mysqli_query($con, "SELECT release_tag, update_time FROM files WHERE id = (SELECT MAX(id) AS id FROM files WHERE release_tag IS NOT NULL && release_tag != '')");
    if ($result && mysqli_num_rows($result) == 1)
    {
      $row = mysqli_fetch_assoc($result);
      if ($row['release_tag'] != "")
      {
        $date_format = "Y-m-d";
        if ($include_time)
          $date_format .= " H:i";
        $tag = $row['release_tag'] . (($include_update_time) ? " (" . date($date_format, $row['update_time']) . ")" : "");
      }
      else
        log_file("Release tag was empty", "../sql.log");
    }
    else
      log_file("Failed to get last tag: " . mysqli_error($con), "../sql.log");
  }
  return $tag;
}

function curl_request_simple($url, $data, $post = false)
{
  return curl_request($url, $data, null, null, $post);
}

/**
 * @param $url string
 * @param $data array Associative array of keys & values
 * @param $username string
 * @param $password string
 * @param $post boolean
 * @return array
 */
function curl_request($url, $data = array(), $username = null, $password = null, $post = false)
{
  $ch = curl_init($url);
  if ($post)
    curl_setopt($ch, CURLOPT_POST, true);
  if (isset($username) && isset($password))
  {
    curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
    curl_setopt($ch, CURLOPT_USERPWD, "$username:$password");
  }
  curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
  curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($data));

  $result = curl_exec($ch);
  $error = curl_error($ch);
  $info = curl_getinfo($ch);
  curl_close($ch);
  return array("result" => $result, "code" => $info['http_code'], "msg" => $error, "url" => $url);
}

/**
 * @param Patreon\SettingSwitch $setting
 * @return string
 */
function generate_settings_switch($setting)
{
  return generate_switch($setting->value, $setting->key, $setting->options);
}

function generate_switch($variable, $key, $options)
{
  $index = 1;
  $str = "";
  foreach ($options as $option => $label)
  {
    if ($index == 1)
      $class = "option_left";
    elseif ($index == count($options))
      $class = "option_right";
    else
      $class = "option_middle";
    $checked = "";
    if (isset($variable) && $variable == $option)
    {
      $class .= " option_active";
      $checked = "checked";
    }
    $str .= "<div class='$class'><label onclick=\"on_options_update(event)\"><input class=\"form-control\" type=\"radio\" autocomplete=\"off\" name=\"$key\" value=\"$option\" $checked />$label</label></div>\n";
    $index++;
  }
  return $str;
}

/**
 * @param \Patreon\Setting|\Patreon\SettingNumber|\Patreon\SettingString|\Patreon\SettingSwitch $setting
 * @return string
 */
function generate_switch_two($setting)
{
  $index = 1;
  $str = "<div class='btn-group btn-group-toggle' role='group' aria-label='" . $setting->title . "'>";
  $counter = 0;
  foreach ($setting->options as $option => $label)
  {
    $sel = $setting->value === $option;
    $str .= "<label class='btn btn-primary " . ($sel ? "active" : "") . "'><input type='radio' name='" . $setting->key . "' id='" . $setting->key . "_$counter' " . ($sel ? "checked" : "") . " >" . $label . "</label>";
    $counter++;
  }
  $str .= "</div>";
  return $str;
}

function send_minecraft_command($command)
{
//  if (test_setting("command_method", "akliz"))
  if (true)
  {
    $server_id = "8763";
    $instance_name = "custom-minecraft";
    $username = "martinek90@hotmail.com";
    $password = "K$%6vwfK$%Qa$9Hx2iNcjd&0F09eqD";
    list($result, $code) = curl_request("https://cc.akliz.net/api/servers/$server_id/processes/$instance_name/input", array( "input" => $command), $username, $password, true);

    if ($result === false)
      return json_encode(array("result" => 2, "msg" => "Http request failed ($code)"));
    elseif ($result == "")
      return json_encode(array("result" => 0, "msg" => "Command sent successfully!"));
    else
      return json_decode($result);
  }
  return json_encode(array("result" => 1, "msg" => "No action taken"));
}

/**
 * @param string $title
 * @param string $message
 * @param string $link
 * @param array $methods
 * @return array
 */
function push_notification($title, $message, $link = null, $methods = array("pushbullet", "rss"))
{
  require_once __DIR__ . "/auth.php";
  require_once __DIR__ . "/classes/SimpleSetting.php";
  $sett = new SimpleSetting();
  $result = array();
  if ($sett->Get(SimpleSetting::$PUSH_NOTIFICATIONS)->TestEnabled())
  {
    global $pushbullet_token, $pushbullet_channel_tag;
    if (in_array("pushbullet", $methods))
    {
      if (isset($pushbullet_token) && isset($pushbullet_channel_tag))
      {
        $ch = curl_init("https://api.pushbullet.com/v2/pushes");
        curl_setopt($ch, CURLOPT_HTTPHEADER, array("Access-Token: $pushbullet_token"));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POST, true);
        if (isset($link))
          curl_setopt($ch, CURLOPT_POSTFIELDS, array("channel_tag" => $pushbullet_channel_tag, "type" => "link", "title" => $title, "body" => $message, "url" => $link));
        else
          curl_setopt($ch, CURLOPT_POSTFIELDS, array("channel_tag" => $pushbullet_channel_tag, "type" => "note", "title" => $title, "body" => $message));
        array_push($result, curl_exec($ch));
      }
      else
        array_push($result, "Unable to note pushbullet. Missing auth.");
    }
  }
  else
    array_push($result, "Notifications are disabled");
  return $result;
}

function role_has_access($role_id, $level = "public")
{
  require_once __DIR__ . "/classes/Role.php";
  $role = new Role();
  if ($level == "public")
  {
    if ($role->role_to_int($role_id) >= $role->role_to_int($role::$PATRON))
      return true;
  }
  if ($level == "mod")
  {
    if ($role->role_to_int($role_id) >= $role->role_to_int($role::$MODERATOR))
      return true;
  }
  if ($level == "admin")
  {
    if ($role->role_to_int($role_id) >= $role->role_to_int($role::$ADMIN))
      return true;
  }
  return false;
}

/**
 * If a valid session cookie is found, and the user has sufficient privileges returns role, else redirects user to main page unless redirect has been set to false
 * Returns email as second return field on success
 * @param $access_level string public|mod|admin
 * @param $allow_redirect bool
 * @return array
 */
function get_login_session($access_level = "public", $allow_redirect = true)
{
  global $con, $instance_prefix, $login_redirect_url;
  require_once __DIR__ . "/internal/inc_connect.php";
  require_once __DIR__ . "/auth.php";
  require_once "inc_common_functions.php";
  if (isset($instance_prefix))
  {
    if (isset($_COOKIE[$instance_prefix . "_foremodpacker_session"]))
    {
      $token = $_COOKIE[$instance_prefix . "_foremodpacker_session"];
      log_file("Session cookie found: $token");
      $query = "SELECT * FROM users WHERE login_token = '$token'";
      $result = mysqli_query($con, $query);
      if (!$result)
      {
        log_file("Failed to execute session query: " . mysqli_error($con));
        if ($allow_redirect)
          header("Location: $login_redirect_url");
        return array(false, "error");
      }
      if (mysqli_num_rows($result) == 0)
      {
        if ($allow_redirect)
          header("Location: $login_redirect_url");
        return array(false, "login");
      }
      $user = mysqli_fetch_assoc($result);
      if ($user['token_expires'] >= time() && role_has_access($user['role'], $access_level))
        return array($user['role'], $user['email']);
      else
      {
        if ($allow_redirect)
          header("Location: $login_redirect_url?error=4");
        return array(false, "unauthorized");
      }
    }
    if ($allow_redirect)
      header("Location: $login_redirect_url");
    return array(false, "login");
  }
  header("Location: $login_redirect_url?error=1");
  return array(false, "error");
}

/**
 * Attempts to init a new session for the
 * @param $email
 * @return bool
 */
function init_login_session($email)
{
  require_once __DIR__ . "/internal/inc_connect.php";
  require_once __DIR__ . "/auth.php";
  global $con, $max_session_length, $instance_prefix;
  if (isset($email))
  {
    $token = uniqid();
    $expire = time() + $max_session_length;
    if (isset($_COOKIE['foremodpacker_keep_me_logged_in']) && $_COOKIE['foremodpacker_keep_me_logged_in'] == "true")
      $expire = time() + 31540000;
    $session_result = mysqli_query($con, "UPDATE users SET login_token = '$token', token_expires = '$expire' WHERE email = '$email'");
    if ($session_result)
    {
//      log_file("Init login session for '$email' expire $max_session_length", "generic.log");
      setcookie($instance_prefix."_foremodpacker_session", $token, $expire);
      return true;
    }
    else
      log_file("SQL Update to create session for '$email' failed: " . mysqli_error($con));
  }
  log_file("No email provided for session init");
  return false;
}

function get_role_from_session()
{
  global $con, $instance_prefix;
  require_once __DIR__ . "/classes/Role.php";
  require_once __DIR__ . "/internal/inc_connect.php";
  if (isset($_COOKIE[$instance_prefix."_foremodpacker_session"]))
  {
    $token = $_COOKIE[$instance_prefix."_foremodpacker_session"];
    $query = "SELECT * FROM users WHERE login_token = '$token'";
    $result = mysqli_query($con, $query);
    if (!$result)
      return "";
    if (mysqli_num_rows($result) == 0)
      return "";
    $user = mysqli_fetch_assoc($result);
    return $user['role'];
  }
  return "";
}

function test_function()
{
  global $login_redirect_url;
  require_once __DIR__ . "/auth.php";
  return $login_redirect_url;
}

/**
 * Menu constructing function
 * $menu = array(
 *  "Item Name" => array(
 *    "type" => menu|link,
 *    "items" => array(
 *      array("type" => link|spacer, "url" => "http://foo.bar")
 *    )
 *  )
 * )
 * @deprecated Use Menu class instead
 */
function build_menu()
{
  global $old_menu, $session;

  function parse_item($label, $item, $item_class = "sub_menu_item")
  {
    global $session;
    if (isset($item['level']) && !role_has_access($session, $item['level']))
      return;
    if ($item['type'] == "spacer")
    {
      echo "<div class=\"menu_spacer\">$label</div>";
    }
    elseif ($item['type'] == "link")
    {
      $url = $item['url'];
      $title = "";
      if (isset($item['title']))
        $title = "title = '" . str_replace("'", "&#39;", str_replace("\"", "&#34;", $item['title'])) . "'";
      $align = "";
      if (isset($item['align']))
        $align = "style=\"float: " . $item['align'] . ";\"";
      echo "<a href=\"$url\" $align><div class=\"$item_class\" $title>$label</div></a>";
    }
    elseif ($item['type'] == "button")
    {
      $onclick = $item['func'];
      echo "<div class=\"$item_class\" onclick=\"$onclick\">$label</div>";
    }
    elseif ($item['type'] == "menu")
    {
      if (isset($item['items']))
      {
        echo "<div class=\"sub_menu\">";
        foreach ($item['items'] as $label => $sub_item)
          parse_item($label, $sub_item);
        echo "</div>";
      }
      else
        echo "<div style='color: red;'>Something went wrong parsing the menu!</div>";
    }
  }

  if (isset($old_menu) && is_array($old_menu) && count($old_menu) > 0)
  {
    echo "<div class=\"menu_bar_spacer\"></div>";
    echo "<div class=\"menu_bar\" style=\"z-index: 1000;\">";
    foreach ($old_menu as $label => $item)
    {
      if (isset($item['level']) && !role_has_access($session, $item['level']))
        continue;
      if ($item['type'] == "menu")
      {
        if (isset($item['items']))
        {
          $length = strlen($label);
          foreach ($item['items'] as $labellabel => $itemitem)
            $length = max($length, strlen($labellabel));
          $width = $length * 0.60 . "em"; //This makes the base menu item wide enough to fit the longest sub-item

          echo "<div class=\"menu_item\" style=\"text-decoration: none; width: $width\">";
          echo $label;
          parse_item($label, $item);
          echo "</div>";
        }
        else
          echo "<div style='color: red;'>Something went wrong parsing the menu!</div>";
      }
      else
        parse_item($label, $item, "menu_item");
    }
    echo "</div>";
  }
}

/**
 * @param $id
 * @param $time_from
 * @param $time_to
 * @return array
 */
function ad_get($id, $time_from, $time_to)
{
  require_once __DIR__ . "/internal/inc_connect.php";
  global $con;

  $query = "";
  if (isset($id))
    $query = "SELECT * FROM ads WHERE id = '$id'";
  elseif (isset($time_from) || isset($time_to))
  {
    $query = "SELECT * FROM ads WHERE";
    if (isset($time_from))
      $query .= " bumped > '$time_from'";
    if (isset($time_to))
    {
      if (isset($time_from))
        $query .= " AND ";
      $query .= " bumped < '$time_to'";
    }
  }
  if ($query != "")
  {
    $result = mysqli_query($con, $query);

    if ($result)
      return mysqli_fetch_all($result);
    else
      return array(false, mysqli_error($con));
  }
  return array(false, "Insufficient parameters specified");
}

/**
 * @param $email
 * @param $type
 * @param $title
 * @param $body
 * @return string
 */
function ad_new($email, $type, $title, $body)
{
  global $con, $max_ad_amount;
  $email = mysqli_escape($con, $email);
  $title = mysqli_escape($con, $title);
  $body = mysqli_escape($con, $body);
  require_once __DIR__ . "/classes/ErrorCodes.php";
  $words = explode(" ", $body);

  if (count($words) > 70)
    return return_error_as_json(ErrorCodes::$AD_TOO_MANY_WORDS, count($words), true);

  require_once __DIR__ . "/constants.php";
  require_once __DIR__ . "/internal/inc_connect.php";
  require_once __DIR__ . "/api/inc_functions.php";

  if (!isset($max_ad_amount))
    return return_error_as_json(ErrorCodes::$UNDEFINED_VARIABLE, "max_ad_amount", true);

  $query = "SELECT * FROM ads WHERE owner_email = '$email'";
  $result = mysqli_query($con, $query);

  if (mysqli_num_rows($result) >= $max_ad_amount)
    return return_error_as_json(ErrorCodes::$AD_LIMIT_REACHED, mysqli_num_rows($result) . " / " . $max_ad_amount, true);

  $time = time();
  $query = "INSERT INTO ads (created_time, bumped, edited, owner_email, type, title, body) VALUES ('$time', '$time', '0', '$email', '$type', '$title', '$body')";

  $result = mysqli_query($con, $query);

  if ($result)
    return return_success_as_json(null, true);
  else
    return return_error_as_json(ErrorCodes::$SQL_ERROR, mysqli_error($con), true);
}

function ad_edit($id, $title, $body)
{
  require_once __DIR__ . "/internal/inc_connect.php";
  global $con;

  $time = time();
  $query = "UPDATE ads SET title = '$title', body = '$body', edited = '$time', bumped = '$time' WHERE id = '$id'";

  $result = mysqli_query($con, $query);

  if ($result)
    return array(true, "");
  else
    return array(false, mysqli_error($con));
}

function ad_delete($id)
{
  require_once __DIR__ . "/internal/inc_connect.php";
  global $con;

  $query = "DELETE FROM ads WHERE id = '$id'";

  $result = mysqli_query($con, $query);

  if ($result)
    return array(true, "");
  else
    return array(false, mysqli_error($con));
}

function ad_bump($id)
{
  require_once __DIR__ . "/internal/inc_connect.php";
  global $con;

  $time = time();
  $query = "UPDATE ads SET bumped = '$time' WHERE id = '$id'";

  $result = mysqli_query($con, $query);

  if ($result)
    return array(true, "");
  else
    return array(false, mysqli_error($con));
}

/**
 * @return Menu
 */
function get_default_menu()
{
  require_once __DIR__ . "/menu/Menu.php";
  require_once __DIR__ . "/menu/MenuButton.php";
  require_once __DIR__ . "/menu/MenuLink.php";
  require_once __DIR__ . "/menu/MenuSpacer.php";
  require_once __DIR__ . "/classes/SimpleSetting.php";
  require_once __DIR__ . "/classes/Role.php";

  $simpleSetting = new SimpleSetting();

  $menu = new Menu();

  $menu->AddLink(new MenuLink("Log Out", "log_out"));

  $subscribe = new Menu("Subscribe to updates"); $menu->AddSubmenu($subscribe);
  $subscribe->AddLink(new MenuLink("Pushbullet", "https://www.pushbullet.com/channel?tag=officialrailcraftpack"));
  $subscribe->AddLink(new MenuLink("RSS", "//towerofawesome.org/ForeModpacker/rss_2.xml"));

  $info = new Menu("Information"); $menu->AddSubmenu($info);
  if ($simpleSetting->Load(SimpleSetting::$PUBLIC_MOD_LIST_ENABLED)->TestEnabled())
    $info->AddLink(new MenuLink("Public Mod List", "public_modlist"));
  $info->AddLink(new MenuLink("Classifieds", "classifieds_list"));
  $info->AddLink(new MenuLink("Changelog", "changelog"));
  $info->AddLink(new MenuLink("Scoreboards", "scoreboards"));
  $info->AddLink(new MenuLink("Public API Documentation", "api"));

  $support = new Menu("Support"); $info->AddSubmenu($support);
  $support->AddLink(new MenuLink("Discord Server", "https://discord.gg/xRk8MnD"));
  $support->AddLink(new MenuLink("Issue Tracker (Modpack)", "https://github.com/Railcraft/RailcraftPatronServer/issues"));
  $support->AddLink(new MenuLink("Issue Tracker (Platform)", "https://github.com/Forecaster/ForeModpacker/issues"));

  $extras = new Menu("Extras"); $menu->AddSubmenu($extras);
  $updaters = new Menu("Updaters"); $extras->AddSubmenu($updaters);
  $winlinmac = new Menu("Windows, Mac & Linux"); $updaters->AddSubmenu($winlinmac);
  $winlinmac->AddLink(new MenuLink("LPCPI", "https://www.theender.net/RCPU/", "Lizzy´s Railcraft Patron Server Pack Updater", null, true));

  $menu->AddLink(new MenuLink("Licenses", "https://docs.google.com/spreadsheets/d/1AHxz1KdAVzV6CiEWW-atoDBGth7GeauheEf95J33YuY/edit#gid=0", "", null, true));
  $menu->AddLink(new MenuLink("Admin", "admin", null, Role::$ADMIN));

  return $menu;
}

/**
 * @return Menu
 */
function get_admin_menu()
{
  require_once __DIR__ . "/menu/Menu.php";
  require_once __DIR__ . "/menu/MenuButton.php";
  require_once __DIR__ . "/menu/MenuLink.php";
  require_once __DIR__ . "/menu/MenuSpacer.php";
  require_once __DIR__ . "/classes/Role.php";

  $menu = new Menu();
  $menu->AddLink(new MenuLink("Back To Modpack", ".."));
  $menu->AddLink(new MenuLink("Log Out", "../log_out"));
  $menu->AddLink(new MenuLink("Users…", "user_management"));

  $pack = new Menu("Pack");
  $menu->AddSubmenu($pack);
  $pack->AddButton(new MenuButton("Generate Server", "show_server_genreate_dialog()"));
  $pack->AddButton(new MenuButton("Generate Client DEV", "request_client_pack(true)", "Generate latest pack including un-tagged"));
  $pack->AddButton(new MenuButton("Generate Client", "request_client_pack()", "Generate pack from latest tag"));
  $pack->AddLink(new MenuLink("Changelog", "../changelog"));
  $pack->AddLink(new MenuLink("Add File…", "add"));

  $server = new Menu("Server");
  $menu->AddSubmenu($server);
  $server->AddLink(new MenuLink("Auto-update Server…", "auto_update", null, Role::$ADMIN));
  $server->AddLink(new MenuLink("Server Messages…", "server_messages"));
  $server->AddLink(new MenuLink("Clear Build History…", "clear_build_history"));

  $menu->AddLink(new MenuLink("Settings…", "settings"));

  return $menu;
}

function get_badge($contents, $style = "primary")
{
  return "<span class='badge badge-" . $style . "'>" . $contents . "</span>";
}