<?

/**
 * Class Menu
 * Menu system version 0.1
 * @property String $title
 * @property Menu[]|MenuLink[]|MenuButton[]|MenuSpacer[] $items;
 * @property int $pointer
 * @property int $menu_level
 */
class Menu
{
  public $title;
  public $items;
  public $pointer;
  public $menu_level;

  /**
   * Menu constructor.
   * @param string $title
   */
  public function __construct($title = "")
  {
    $this->title = $title;
    $this->items = array();
    $this->pointer = 0;
  }

  /**
   * @return String
   */
  public function GetTitle()
  {
    return $this->title;
  }

  /**
   * @return Menu[]|MenuButton[]|MenuLink[]|MenuSpacer[]
   */
  public function GetItems()
  {
    return $this->items;
  }

  /**
   * @param Menu $submenu
   * @param bool $first
   */
  public function AddSubmenu($submenu, $first = false)
  {
    if (!$first)
      array_push($this->items, $submenu);
    else
      array_unshift($this->items, $submenu);
  }

  /**
   * @param MenuLink $link
   * @param bool $first
   */
  public function AddLink($link, $first = false)
  {
    if (!$first)
      array_push($this->items, $link);
    else
      array_unshift($this->items, $link);
  }

  /**
   * @param MenuButton $button
   * @param bool $first
   */
  public function AddButton($button, $first = false)
  {
    if (!$first)
      array_push($this->items, $button);
    else
      array_unshift($this->items, $button);
  }

  /**
   * @param MenuSpacer $spacer
   * @param bool $first
   */
  public function AddSpacer($spacer, $first = false)
  {
    if (!$first)
      array_push($this->items, $spacer);
    else
      array_unshift($this->items, $spacer);
  }

  /**
   * @param int $level
   * @param int $width
   * @param int $zIndex
   * @return string
   */
  public function BuildMenu($level = 0, $width = 0, $zIndex = 2000)
  {
    global $menu_root;
    $class = "menu_item";
    if ($level > 0)
      $class = "sub_menu_item";
    $str ="<link rel='stylesheet' href='" . $menu_root . "menu/MenuStyles.css' />";
    if ($level === 0)
    {
      $str .= "<div class=\"menu_bar_spacer\"></div>";
      $str .= "<div class=\"menu_bar\" style=\"z-index: 1000;\">";
    }
    else
    {
      //<editor-fold desc="Label length handling">
      $length = strlen($this->GetTitle());
      foreach ($this->GetItems() as $it)
        $length = max($length, strlen($it->GetTitle()));
      $width = max($width, ($length * 0.60) + 1); //This makes the base menu item wide enough to fit the longest sub-item, or use the parent width if wider
      //</editor-fold>

      //<editor-fold desc="Sub-menu offset">
      $offset = "";
      if ($level > 1)
        $offset = "style='left: 20%'";
      //</editor-fold>

      $str .= "<div class=\"menu_item\" style=\"text - decoration: none; width: ".$width."em; z-index: $zIndex;\">" . ($level > 1 ? " > " : "") . $this->GetTitle() . "<div class='sub_menu' $offset>";
    }

    foreach ($this->items as $item)
    {
      $zIndex--;
      if (is_a($item, "MenuButton"))
        /** @var MenuButton $item */
        $str .= $item->BuildItem($class, $zIndex);
      elseif (is_a($item, "MenuLink"))
        /** @var MenuLink $item */
        $str .= $item->BuildItem($class, $zIndex);
      elseif (is_a($item, "MenuSpacer"))
        /** @var MenuSpacer $item */
        $str .= $item->BuildItem();
      elseif (is_a($item, "Menu"))
        /** @var Menu $item */
        $str .= $item->BuildMenu($level + 1, $width, $zIndex);
      else
        $str .= "<div>Unknown item '" . $item->GetTitle() . "'</div>";
    }
    if ($level === 0)
      $str .= "</div>";
    else
      $str .= "</div></div>";

    return $str;
  }

  public function RemoteItem($label)
  {
  }
}